// Code generated by modernc.org/undup from the per-target sqlite_*.go files; DO NOT EDIT.

//go:build (freebsd && 386) || (freebsd && amd64) || (freebsd && arm) || (freebsd && arm64) || (linux && 386) || (linux && amd64) || (linux && arm) || (linux && arm64) || (linux && loong64) || (linux && ppc64le) || (linux && riscv64) || (linux && s390x) || (netbsd && amd64) || (openbsd && amd64) || (openbsd && arm64) || (windows && (amd64 || arm64)) || (windows && 386)

package sqlite3

const SQLITE_ENABLE_LOCKING_STYLE = 0

const TIMER_ABSTIME = 1

const _XOPEN_SOURCE = 600

const __FLT_MIN__ = 0

const __LDBL_MIN__ = 0

var _aMap = [5]uint8{
	0: uint8(SQLITE_ROLLBACK),
	1: uint8(SQLITE_ABORT),
	2: uint8(SQLITE_FAIL),
	3: uint8(SQLITE_IGNORE),
	4: uint8(SQLITE_REPLACE),
}
