package git

import "github.com/aymanbagabas/git-module"

// CommandOptions contain options for running a git command.
type CommandOptions = git.CommandOptions

// CloneOptions contain options for cloning a repository.
type CloneOptions = git.CloneOptions
