// Code generated by modernc.org/undup from the per-target sqlite_*.go files; DO NOT EDIT.

//go:build (linux && 386) || (linux && amd64) || (netbsd && amd64) || (windows && (amd64 || arm64)) || (windows && 386)

package sqlite3

const __ATOMIC_HLE_ACQUIRE = 65536

const __ATOMIC_HLE_RELEASE = 131072

const __LONG_DOUBLE_64__ = 1
