// Code generated by modernc.org/undup from the per-target sqlite_*.go files; DO NOT EDIT.

//go:build (linux && 386) || (linux && amd64) || (linux && arm) || (linux && arm64) || (linux && loong64) || (linux && ppc64le) || (linux && riscv64)

package sqlite3

const __BYTE_ORDER = 1234

const __GNUC_WIDE_EXECUTION_CHARSET_NAME = "UTF-32LE"
