// Code generated by modernc.org/undup from the per-target sqlite_*.go files; DO NOT EDIT.

//go:build (linux && 386) || (linux && amd64) || (linux && arm) || (linux && arm64) || (linux && loong64) || (linux && ppc64le) || (linux && riscv64) || (linux && s390x) || (windows && (amd64 || arm64)) || (windows && 386)

package sqlite3

const CLOCK_MONOTONIC = 1

const CLOCK_THREAD_CPUTIME_ID = 3

const EAGAIN = 11

const SQLITE_THREADS_IMPLEMENTED = 1

const WINT_MIN = 0

const _POSIX_MONOTONIC_CLOCK = 200809

const _POSIX_TIMERS = 200809

const __GNUC_EXECUTION_CHARSET_NAME = "UTF-8"

const __WINT_MIN__ = 0
