// Code generated by modernc.org/undup from the per-target sqlite_*.go files; DO NOT EDIT.

//go:build (darwin && amd64) || (darwin && arm64) || (freebsd && amd64) || (freebsd && arm) || (freebsd && arm64) || (linux && amd64) || (linux && arm64) || (linux && riscv64) || (openbsd && amd64) || (openbsd && arm64)

package sqlite3

const __FLT16_DECIMAL_DIG__ = 5

const __FLT16_DENORM_MIN__ = 0

const __FLT16_DIG__ = 3

const __FLT16_EPSILON__ = 0

const __FLT16_HAS_DENORM__ = 1

const __FLT16_HAS_INFINITY__ = 1

const __FLT16_HAS_QUIET_NAN__ = 1

const __FLT16_MANT_DIG__ = 11

const __FLT16_MAX_10_EXP__ = 4

const __FLT16_MAX_EXP__ = 16

const __FLT16_MAX__ = 0

const __FLT16_MIN_10_EXP__ = -4

const __FLT16_MIN_EXP__ = -13

const __FLT16_MIN__ = 0

const __FLT16_NORM_MAX__ = 0
