// Code generated by modernc.org/undup from the per-target sqlite_*.go files; DO NOT EDIT.

//go:build (linux && 386) || (linux && amd64) || (linux && ppc64le) || (linux && s390x) || (windows && (amd64 || arm64)) || (windows && 386)

package sqlite3

const __DEC128_EPSILON__ = 0

const __DEC128_MANT_DIG__ = 34

const __DEC128_MAX_EXP__ = 6145

const __DEC128_MAX__ = 0

const __DEC128_MIN_EXP__ = -6142

const __DEC128_MIN__ = 0

const __DEC128_SUBNORMAL_MIN__ = 0

const __DEC32_EPSILON__ = 0

const __DEC32_MANT_DIG__ = 7

const __DEC32_MAX_EXP__ = 97

const __DEC32_MAX__ = 0

const __DEC32_MIN_EXP__ = -94

const __DEC32_MIN__ = 0

const __DEC32_SUBNORMAL_MIN__ = 0

const __DEC64_EPSILON__ = 0

const __DEC64_MANT_DIG__ = 16

const __DEC64_MAX_EXP__ = 385

const __DEC64_MAX__ = 0

const __DEC64_MIN_EXP__ = -382

const __DEC64_MIN__ = 0

const __DEC64_SUBNORMAL_MIN__ = 0
