// Code generated by modernc.org/undup from the per-target sqlite_*.go files; DO NOT EDIT.

//go:build (darwin && amd64) || (darwin && arm64) || (freebsd && 386) || (freebsd && amd64) || (freebsd && arm64) || (linux && 386) || (linux && amd64) || (linux && arm64) || (linux && loong64) || (linux && ppc64le) || (linux && riscv64) || (linux && s390x) || (netbsd && amd64) || (openbsd && amd64) || (openbsd && arm64)

package sqlite3

import (
	"modernc.org/libc"
)

type Titimerval = struct {
	Fit_interval Ttimeval
	Fit_value    Ttimeval
}

/*
** An instance of the following structure serves as the key used
** to locate a particular unixInodeInfo object.
 */
type TunixFileId = struct {
	Fdev Tdev_t
	Fino Tu64
}

var _aFlagOp = [21]struct {
	Fop   int32
	Fmask Tu64
}{
	0: {
		Fop:   int32(SQLITE_DBCONFIG_ENABLE_FKEY),
		Fmask: uint64(SQLITE_ForeignKeys),
	},
	1: {
		Fop:   int32(SQLITE_DBCONFIG_ENABLE_TRIGGER),
		Fmask: uint64(SQLITE_EnableTrigger),
	},
	2: {
		Fop:   int32(SQLITE_DBCONFIG_ENABLE_VIEW),
		Fmask: uint64(SQLITE_EnableView),
	},
	3: {
		Fop:   int32(SQLITE_DBCONFIG_ENABLE_FTS3_TOKENIZER),
		Fmask: uint64(SQLITE_Fts3Tokenizer),
	},
	4: {
		Fop:   int32(SQLITE_DBCONFIG_ENABLE_LOAD_EXTENSION),
		Fmask: uint64(SQLITE_LoadExtension),
	},
	5: {
		Fop:   int32(SQLITE_DBCONFIG_NO_CKPT_ON_CLOSE),
		Fmask: uint64(SQLITE_NoCkptOnClose),
	},
	6: {
		Fop:   int32(SQLITE_DBCONFIG_ENABLE_QPSG),
		Fmask: uint64(SQLITE_EnableQPSG),
	},
	7: {
		Fop:   int32(SQLITE_DBCONFIG_TRIGGER_EQP),
		Fmask: uint64(SQLITE_TriggerEQP),
	},
	8: {
		Fop:   int32(SQLITE_DBCONFIG_RESET_DATABASE),
		Fmask: uint64(SQLITE_ResetDatabase),
	},
	9: {
		Fop:   int32(SQLITE_DBCONFIG_DEFENSIVE),
		Fmask: uint64(SQLITE_Defensive),
	},
	10: {
		Fop:   int32(SQLITE_DBCONFIG_WRITABLE_SCHEMA),
		Fmask: libc.Uint64FromInt32(libc.Int32FromInt32(SQLITE_WriteSchema) | libc.Int32FromInt32(SQLITE_NoSchemaError)),
	},
	11: {
		Fop:   int32(SQLITE_DBCONFIG_LEGACY_ALTER_TABLE),
		Fmask: uint64(SQLITE_LegacyAlter),
	},
	12: {
		Fop:   int32(SQLITE_DBCONFIG_DQS_DDL),
		Fmask: uint64(SQLITE_DqsDDL),
	},
	13: {
		Fop:   int32(SQLITE_DBCONFIG_DQS_DML),
		Fmask: uint64(SQLITE_DqsDML),
	},
	14: {
		Fop:   int32(SQLITE_DBCONFIG_LEGACY_FILE_FORMAT),
		Fmask: uint64(SQLITE_LegacyFileFmt),
	},
	15: {
		Fop:   int32(SQLITE_DBCONFIG_TRUSTED_SCHEMA),
		Fmask: uint64(SQLITE_TrustedSchema),
	},
	16: {
		Fop:   int32(SQLITE_DBCONFIG_STMT_SCANSTATUS),
		Fmask: uint64(SQLITE_StmtScanStatus),
	},
	17: {
		Fop:   int32(SQLITE_DBCONFIG_REVERSE_SCANORDER),
		Fmask: uint64(SQLITE_ReverseOrder),
	},
	18: {
		Fop:   int32(SQLITE_DBCONFIG_ENABLE_ATTACH_CREATE),
		Fmask: libc.Uint64FromInt32(libc.Int32FromInt32(0x00010)) << libc.Int32FromInt32(32),
	},
	19: {
		Fop:   int32(SQLITE_DBCONFIG_ENABLE_ATTACH_WRITE),
		Fmask: libc.Uint64FromInt32(libc.Int32FromInt32(0x00020)) << libc.Int32FromInt32(32),
	},
	20: {
		Fop:   int32(SQLITE_DBCONFIG_ENABLE_COMMENTS),
		Fmask: libc.Uint64FromInt32(libc.Int32FromInt32(0x00040)) << libc.Int32FromInt32(32),
	},
}
