// Code generated by modernc.org/undup from the per-target sqlite_*.go files; DO NOT EDIT.

//go:build (linux && 386) || (linux && amd64) || (linux && arm64) || (windows && (amd64 || arm64)) || (windows && 386)

package sqlite3

const __FLT128_IS_IEC_60559__ = 2

const __FLT64X_IS_IEC_60559__ = 2
