// Code generated by modernc.org/undup from the per-target sqlite_*.go files; DO NOT EDIT.

//go:build (linux && 386) || (linux && amd64) || (linux && arm) || (linux && arm64) || (linux && loong64) || (linux && riscv64) || (linux && s390x)

package sqlite3

const EDEADLOCK = 35

const F2FS_IOC_ABORT_VOLATILE_WRITE = 62725

const F2FS_IOC_COMMIT_ATOMIC_WRITE = 62722

const F2FS_IOC_GET_FEATURES = 2147546380

const F2FS_IOC_START_ATOMIC_WRITE = 62721

const F2FS_IOC_START_VOLATILE_WRITE = 62723

const FIOASYNC = 21586

const FIOCLEX = 21585

const FIONBIO = 21537

const FIONCLEX = 21584

const FIONREAD = 21531

const MAP_LOCKED = 8192

const MAP_NORESERVE = 16384

const MCL_ONFAULT = 4

const TCFLSH = 21515

const TCGETA = 21509

const TCGETS = 21505

const TCGETX = 21554

const TCSBRK = 21513

const TCSETA = 21510

const TCSETAF = 21512

const TCSETAW = 21511

const TCSETS = 21506

const TCSETSF = 21508

const TCSETSW = 21507

const TCSETX = 21555

const TCSETXF = 21556

const TCSETXW = 21557

const TCXONC = 21514

const TIOCGDEV = 2147767346

const TIOCGEXCL = 2147767360

const TIOCGISO7816 = 2150126658

const TIOCGPGRP = 21519

const TIOCGPKT = 2147767352

const TIOCGPTLCK = 2147767353

const TIOCGPTN = 2147767344

const TIOCGPTPEER = 21569

const TIOCGWINSZ = 21523

const TIOCINQ = 21531

const TIOCOUTQ = 21521

const TIOCSIG = 1074025526

const TIOCSISO7816 = 3223868483

const TIOCSPGRP = 21520

const TIOCSPTLCK = 1074025521

const TIOCSWINSZ = 21524

const _IOC_NONE = 0

const _IOC_WRITE = 1
