// Code generated by modernc.org/undup from the per-target sqlite_*.go files; DO NOT EDIT.

//go:build (linux && 386) || (linux && amd64) || (linux && loong64) || (linux && riscv64) || (linux && s390x)

package sqlite3

const O_DIRECT = 16384

const O_DIRECTORY = 65536

const O_LARGEFILE = 32768

const O_NOFOLLOW = 131072

const O_TMPFILE = 4259840
