// Code generated by modernc.org/undup from the per-target sqlite_*.go files; DO NOT EDIT.

//go:build (darwin && amd64) || (freebsd && amd64) || (linux && amd64) || (netbsd && amd64) || (openbsd && amd64)

package sqlite3

const __MMX__ = 1

const __SSE2_MATH__ = 1

const __SSE2__ = 1

const __SSE_MATH__ = 1

const __SSE__ = 1
