// Code generated by modernc.org/undup from the per-target sqlite_*.go files; DO NOT EDIT.

//go:build (darwin && amd64) || (darwin && arm64) || (freebsd && 386) || (freebsd && amd64) || (freebsd && arm) || (freebsd && arm64) || (linux && 386) || (linux && amd64) || (linux && arm) || (linux && arm64) || (linux && loong64) || (linux && ppc64le) || (linux && riscv64) || (linux && s390x) || (netbsd && amd64)

package sqlite3

import (
	"modernc.org/libc"
)

const MS_INVALIDATE = 2

type Tpthread_t = uintptr

const __restrict = 0

// C documentation
//
//	/*
//	** Return the system page size.
//	**
//	** This function should not be called directly by other code in this file.
//	** Instead, it should be called via macro osGetpagesize().
//	*/
func _unixGetpagesize(tls *libc.TLS) (r int32) {
	return int32(libc.Xsysconf(tls, int32(_SC_PAGESIZE)))
}

/*
** Constants used for locking
 */

type pthread_attr_t = Tpthread_attr_t

type pthread_cond_t = Tpthread_cond_t

type pthread_condattr_t = Tpthread_condattr_t

type pthread_key_t = Tpthread_key_t

type pthread_mutex_t = Tpthread_mutex_t

type pthread_mutexattr_t = Tpthread_mutexattr_t

type pthread_once_t = Tpthread_once_t

type pthread_rwlock_t = Tpthread_rwlock_t

type pthread_rwlockattr_t = Tpthread_rwlockattr_t

type pthread_t = Tpthread_t
