// Code generated by modernc.org/undup from the per-target sqlite_*.go files; DO NOT EDIT.

//go:build (freebsd && 386) || (freebsd && amd64) || (freebsd && arm) || (freebsd && arm64) || (linux && 386) || (linux && amd64) || (linux && arm) || (linux && arm64) || (linux && loong64) || (linux && ppc64le) || (linux && riscv64) || (linux && s390x)

package sqlite3

const F_SEAL_GROW = 4

const F_SEAL_SEAL = 1

const F_SEAL_SHRINK = 2

const F_SEAL_WRITE = 8

const MFD_ALLOW_SEALING = 2

const MFD_CLOEXEC = 1

const MFD_HUGETLB = 4

const RTLD_DI_LINKMAP = 2

const SEEK_DATA = 3

const SEEK_HOLE = 4

type Tcookie_io_functions_t = struct {
	Fread   uintptr
	Fwrite  uintptr
	Fseek   uintptr
	Fclose1 uintptr
}

type Ttimer_t = uintptr

const _PC_REC_INCR_XFER_SIZE = 14

const _PC_REC_MAX_XFER_SIZE = 15

const _PC_REC_MIN_XFER_SIZE = 16

const _PC_REC_XFER_ALIGN = 17

type cookie_io_functions_t = Tcookie_io_functions_t

type uintmax_t = Tuintmax_t
