// Code generated by modernc.org/undup from the per-target sqlite_*.go files; DO NOT EDIT.

//go:build (linux && 386) || (linux && amd64) || (linux && arm) || (linux && arm64) || (linux && loong64) || (linux && ppc64le) || (linux && riscv64) || (linux && s390x) || (openbsd && amd64) || (openbsd && arm64)

package sqlite3

const EILSEQ = 84

const _PC_FILESIZEBITS = 13

const _PC_SYMLINK_MAX = 19

const _POSIX2_VERSION = 200809

const _POSIX_SAVED_IDS = 1

const _POSIX_SHARED_MEMORY_OBJECTS = 200809

const _SC_THREAD_ATTR_STACKADDR = 77

const _SC_THREAD_ATTR_STACKSIZE = 78
