// Code generated by modernc.org/undup from the per-target sqlite_*.go files; DO NOT EDIT.

//go:build (linux && amd64) || (netbsd && amd64) || (windows && (amd64 || arm64))

package sqlite3

const __SIZEOF_FLOAT80__ = 16
