// Code generated by modernc.org/undup from the per-target sqlite_*.go files; DO NOT EDIT.

//go:build (freebsd && 386) || (freebsd && amd64) || (freebsd && arm) || (freebsd && arm64) || (linux && 386) || (linux && amd64) || (linux && arm) || (linux && arm64) || (linux && loong64) || (linux && ppc64le) || (linux && riscv64) || (linux && s390x) || (netbsd && amd64)

package sqlite3

const AT_SYMLINK_FOLLOW = 1024

const POSIX_FADV_NORMAL = 0

const POSIX_FADV_RANDOM = 1

const POSIX_FADV_SEQUENTIAL = 2

const POSIX_FADV_WILLNEED = 3

const RTLD_NODELETE = 4096

type Tdev_t = uint64

type pthread_barrier_t = Tpthread_barrier_t

type pthread_barrierattr_t = Tpthread_barrierattr_t

type pthread_spinlock_t = Tpthread_spinlock_t
