// Code generated by modernc.org/undup from the per-target sqlite_*.go files; DO NOT EDIT.

//go:build (linux && 386) || (linux && amd64) || (linux && arm) || (linux && arm64) || (linux && loong64) || (linux && ppc64le) || (linux && riscv64) || (linux && s390x) || (netbsd && amd64) || (windows && (amd64 || arm64)) || (windows && 386)

package sqlite3

const __DEC_EVAL_METHOD__ = 2

const __FLT32X_DECIMAL_DIG__ = 17

const __FLT32X_DENORM_MIN__ = 0

const __FLT32X_DIG__ = 15

const __FLT32X_EPSILON__ = 0

const __FLT32X_HAS_DENORM__ = 1

const __FLT32X_HAS_INFINITY__ = 1

const __FLT32X_HAS_QUIET_NAN__ = 1

const __FLT32X_MANT_DIG__ = 53

const __FLT32X_MAX_10_EXP__ = 308

const __FLT32X_MAX_EXP__ = 1024

const __FLT32X_MAX__ = 0

const __FLT32X_MIN_10_EXP__ = -307

const __FLT32X_MIN_EXP__ = -1021

const __FLT32X_MIN__ = 0

const __FLT32X_NORM_MAX__ = 0

const __FLT32_DECIMAL_DIG__ = 9

const __FLT32_DENORM_MIN__ = 0

const __FLT32_DIG__ = 6

const __FLT32_EPSILON__ = 0

const __FLT32_HAS_DENORM__ = 1

const __FLT32_HAS_INFINITY__ = 1

const __FLT32_HAS_QUIET_NAN__ = 1

const __FLT32_MANT_DIG__ = 24

const __FLT32_MAX_10_EXP__ = 38

const __FLT32_MAX_EXP__ = 128

const __FLT32_MAX__ = 0

const __FLT32_MIN_10_EXP__ = -37

const __FLT32_MIN_EXP__ = -125

const __FLT32_MIN__ = 0

const __FLT32_NORM_MAX__ = 0

const __FLT64_DECIMAL_DIG__ = 17

const __FLT64_DENORM_MIN__ = 0

const __FLT64_DIG__ = 15

const __FLT64_EPSILON__ = 0

const __FLT64_HAS_DENORM__ = 1

const __FLT64_HAS_INFINITY__ = 1

const __FLT64_HAS_QUIET_NAN__ = 1

const __FLT64_MANT_DIG__ = 53

const __FLT64_MAX_10_EXP__ = 308

const __FLT64_MAX_EXP__ = 1024

const __FLT64_MAX__ = 0

const __FLT64_MIN_10_EXP__ = -307

const __FLT64_MIN_EXP__ = -1021

const __FLT64_MIN__ = 0

const __FLT64_NORM_MAX__ = 0

const __GCC_IEC_559 = 2

const __GCC_IEC_559_COMPLEX = 2

const __LONG_LONG_WIDTH__ = 64

const __SCHAR_WIDTH__ = 8

const __SIG_ATOMIC_MIN__ = -2147483648

const __SIG_ATOMIC_TYPE__ = 0
