// Code generated by modernc.org/undup from the per-target sqlite_*.go files; DO NOT EDIT.

//go:build (darwin && amd64) || (darwin && arm64) || (freebsd && 386) || (freebsd && amd64) || (freebsd && arm) || (freebsd && arm64) || (linux && 386) || (linux && amd64) || (linux && arm) || (linux && arm64) || (linux && loong64) || (linux && ppc64le) || (linux && riscv64) || (linux && s390x) || (netbsd && amd64) || (windows && (amd64 || arm64)) || (windows && 386)

package sqlite3

import (
	"modernc.org/libc"
)

const M_1_PI = 0

const M_2_PI = 0

const M_2_SQRTPI = 0

const M_E = 0

const M_LN10 = 0

const M_LN2 = 0

const M_LOG10E = 0

const M_LOG2E = 0

const M_PI = 3.141592653589793

const M_PI_2 = 0

const M_PI_4 = 0

const M_SQRT1_2 = 0

const M_SQRT2 = 0

// C documentation
//
//	/*
//	** Implementation of 0-argument pi() function.
//	*/
func _piFunc(tls *libc.TLS, context uintptr, argc int32, argv uintptr) {
	_ = argv
	Xsqlite3_result_double(tls, context, float64(3.141592653589793))
}
