// Code generated by modernc.org/undup from the per-target sqlite_*.go files; DO NOT EDIT.

//go:build (darwin && amd64) || (freebsd && amd64) || (linux && amd64) || (netbsd && amd64) || (openbsd && amd64) || (windows && (amd64 || arm64))

package sqlite3

const __FXSR__ = 1

const __amd64 = 1

const __amd64__ = 1

const __x86_64 = 1

const __x86_64__ = 1
