// Code generated by modernc.org/undup from the per-target sqlite_*.go files; DO NOT EDIT.

//go:build (darwin && amd64) || (darwin && arm64) || (freebsd && 386) || (freebsd && amd64) || (freebsd && arm) || (linux && 386) || (linux && amd64) || (linux && arm) || (linux && ppc64le) || (linux && s390x) || (netbsd && amd64) || (openbsd && amd64) || (windows && (amd64 || arm64)) || (windows && 386)

package sqlite3

const __DECIMAL_DIG__ = 17

const __LDBL_DECIMAL_DIG__ = 17

const __LDBL_DIG__ = 15

const __LDBL_MANT_DIG__ = 53

const __LDBL_MAX_10_EXP__ = 308

const __LDBL_MAX_EXP__ = 1024

const __LDBL_MIN_10_EXP__ = -307

const __LDBL_MIN_EXP__ = -1021
