// Code generated by modernc.org/undup from the per-target sqlite_*.go files; DO NOT EDIT.

//go:build (darwin && amd64) || (darwin && arm64) || (linux && 386) || (linux && amd64) || (linux && arm) || (linux && arm64) || (linux && loong64) || (linux && ppc64le) || (linux && riscv64) || (linux && s390x)

package sqlite3

const FP_ILOGBNAN = -2147483648

const HAVE_PREAD = 1

const HAVE_PWRITE = 1

type Tiovec = struct {
	Fiov_base uintptr
	Fiov_len  Tsize_t
}

const USE_PREAD = 1

const WNOHANG = 1

const WUNTRACED = 2

const _XOPEN_ENH_I18N = 1

const _XOPEN_UNIX = 1

type iovec = Tiovec
