// Code generated by modernc.org/undup from the per-target sqlite_*.go files; DO NOT EDIT.

//go:build (linux && 386) || (linux && amd64) || (linux && arm) || (linux && arm64) || (linux && loong64) || (linux && ppc64le) || (linux && riscv64) || (linux && s390x) || (netbsd && amd64)

package sqlite3

const FP_SUBNORMAL = 3

const MS_SYNC = 4

const UTIME_NOW = 1073741823

const UTIME_OMIT = 1073741822

const _POSIX_SPAWN = 200809
