// Code generated by modernc.org/undup from the per-target sqlite_*.go files; DO NOT EDIT.

//go:build (linux && 386) || (linux && amd64) || (linux && arm64) || (linux && loong64) || (linux && ppc64le) || (linux && riscv64) || (netbsd && amd64) || (windows && (amd64 || arm64)) || (windows && 386)

package sqlite3

const __FLT128_DECIMAL_DIG__ = 36

const __FLT128_DENORM_MIN__ = 0

const __FLT128_DIG__ = 33

const __FLT128_EPSILON__ = 0

const __FLT128_HAS_DENORM__ = 1

const __FLT128_HAS_INFINITY__ = 1

const __FLT128_HAS_QUIET_NAN__ = 1

const __FLT128_MANT_DIG__ = 113

const __FLT128_MAX_10_EXP__ = 4932

const __FLT128_MAX_EXP__ = 16384

const __FLT128_MAX__ = 0

const __FLT128_MIN_10_EXP__ = -4931

const __FLT128_MIN_EXP__ = -16381

const __FLT128_MIN__ = 0

const __FLT128_NORM_MAX__ = 0

const __FLT64X_DECIMAL_DIG__ = 36

const __FLT64X_DENORM_MIN__ = 0

const __FLT64X_DIG__ = 33

const __FLT64X_EPSILON__ = 0

const __FLT64X_HAS_DENORM__ = 1

const __FLT64X_HAS_INFINITY__ = 1

const __FLT64X_HAS_QUIET_NAN__ = 1

const __FLT64X_MANT_DIG__ = 113

const __FLT64X_MAX_10_EXP__ = 4932

const __FLT64X_MAX_EXP__ = 16384

const __FLT64X_MAX__ = 0

const __FLT64X_MIN_10_EXP__ = -4931

const __FLT64X_MIN_EXP__ = -16381

const __FLT64X_MIN__ = 0

const __FLT64X_NORM_MAX__ = 0
